@@ -1,5 +1,4 @@
 #include <unistd.h>
-#include <errno.h>
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -13,11 +12,13 @@ static void usage(char *name)
 int main(int argc, char *argv[])
 {
 	char *buf = NULL;
+	const char *p;
+	int c;
 	int bufsz = 0;
 	int cmd = 3;	/* Read all messages remaining in the ring buffer */
 	int len = 0;
 	int opt;
-	int i = 0;
+	int newline;
 
 	while ((opt = getopt(argc, argv, "c")) != -1) {
 		switch (opt) {
@@ -33,7 +34,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (!bufsz) {
-		len = klogctl(10, NULL, 0);	/* Return size of the log buffer */
+		len = klogctl(10, NULL, 0); /* Get size of log buffer */
 		if (len > 0)
 			bufsz = len;
 	}
@@ -50,22 +51,28 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	while (buf[i] && i < len)
-		switch (buf[i]) {
+	newline = 1;
+	p = buf;
+	while ((c = *p)) {
+		switch (c) {
+		case '\n':
+			newline = 1;
+			putchar(c);
+			p++;
+			break;
 		case '<':
-			if (i == 0 || buf[i-1] == '\n')
-				i++;
-		case '0' ... '9':
-			if (i > 0 && buf[i-1] == '<')
-				i++;
-		case '>':
-			if (i > 0 && isdigit(buf[i-1]))
-				i++;
+			if (newline && isdigit(p[1]) && p[2] == '>') {
+				p += 3;
+				break;
+			}
+			/* else fall through */
 		default:
-			putchar(buf[i++]);
+			newline = 0;
+			putchar(c);
+			p++;
 		}
-
-	if (buf[i-1] != '\n')
+	}
+	if (!newline)
 		putchar('\n');
 
 	return 0;
